/*
 * Copyright(c) 2006 to 2020 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
 */
#include <cstdlib>
#include <iostream>
#include <chrono>
#include <thread>

/* Include the C++ DDS API. */
#include "dds/dds.hpp"

/* Include data type and specific traits to be used with the C++ DDS API. */
#include "HelloWorldData.hpp"

using namespace org::eclipse::cyclonedds;

  class ExampleListener : public virtual dds::pub::DataWriterListener<HelloWorldData::Msg> {
  public:
      virtual void on_offered_deadline_missed (
          dds::pub::DataWriter<HelloWorldData::Msg>& writer,
          const dds::core::status::OfferedDeadlineMissedStatus& status)
      {
          std::cout << "on_offered_deadline_missed" << std::endl;
      }
 
      virtual void on_offered_incompatible_qos (
          dds::pub::DataWriter<HelloWorldData::Msg>& writer,
          const dds::core::status::OfferedIncompatibleQosStatus& status)
      {
          std::cout << "on_offered_incompatible_qos" << std::endl;
      }
 
      virtual void on_liveliness_lost (
          dds::pub::DataWriter<HelloWorldData::Msg>& writer,
          const dds::core::status::LivelinessLostStatus& status)
      {
          std::cout << "on_liveliness_lost" << std::endl;
      }
 
      virtual void on_publication_matched (
          dds::pub::DataWriter<HelloWorldData::Msg>& writer,
          const dds::core::status::PublicationMatchedStatus& status)
      {
          std::cout << "on_publication_matched, current_count: " << status.current_count() << std::endl;
      }
  };


int main() {
    try {
        std::cout << "=== [Publisher] Create writer." << std::endl;

        dds::domain::DomainParticipant participant(domain::default_id());

        dds::topic::Topic<HelloWorldData::Msg> topic(participant, "HelloWorldData_Msg");

        dds::pub::Publisher publisher(participant);

        dds::pub::DataWriter<HelloWorldData::Msg> writer(
            publisher,
            topic,
            publisher.default_datawriter_qos(),
            new ExampleListener(),
            dds::core::status::StatusMask::all());

        while (writer.publication_matched_status().current_count() == 0) {
            std::this_thread::sleep_for(std::chrono::milliseconds(20));
        }

        HelloWorldData::Msg msg(1, "Hello World");

        std::cout << "=== [Publisher] Write sample." << std::endl;
        writer.write(msg);
        std::cout << "=== [Publisher] Waiting for sample to be accepted." << std::endl;
        while (writer.publication_matched_status().current_count() > 0) {
            std::this_thread::sleep_for(std::chrono::milliseconds(50));
        }

		// dont exit too fast
        int ccc =0;
        while(ccc < 6) {
            std::this_thread::sleep_for(std::chrono::milliseconds(1000));
            ccc++;
        }

    }
    catch (const dds::core::Exception& e) {
        std::cerr << "=== [Publisher] Exception: " << e.what() << std::endl;
        return EXIT_FAILURE;
    }

    std::cout << "=== [Publisher] Done." << std::endl;

    return EXIT_SUCCESS;
}